<?php

/**
 * @file Do Admin settings
 */

/**
 * Admin settings form
 */
function do_form_settings() {
  drupal_add_js(drupal_get_path('module', 'do') . '/js/do.admin.js');
  $form = array();

  $form['do_shortcut'] = array(
    '#type' => 'fieldset',
    '#title' => t('Keyboard shortcut'),
    '#description' => t('Select a keyboard modifier and trigger to launch Do. Hold the modifier and press the trigger to launch Do. The default is Ctrl + Space.'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE
  );

  $form['do_shortcut']['do_modifier'] = array(
    '#type' => 'radios',
    '#name' => 'do_modifier',
    '#title' => t('Keyboard Modifier'),
    '#description' => t('Hold the modifier key to start the shortcut'),
    '#required' => TRUE,
    '#default_value' => variable_get('do_modifier', 'ctrl'),
    '#options' => array('ctrl' => 'ctrl', 'shift' => 'shift', 'alt' => 'alt', 'meta' => 'meta'),
  );

  $form['do_shortcut']['do_trigger'] = array(
    '#type' => 'textfield',
    '#name' => 'do_trigger',
    '#title' => t('Keyboard Trigger'),
    '#description' => t('Press the trigger key whilst holding the modifier key to activate Do.'),
    '#required' => TRUE,
    '#default_value' => variable_get('do_trigger', '32'),
    '#maxlength' => 20,
  );

  $form['do_drush'] = array(
    '#type' => 'fieldset',
    '#title' => t('Drush'),
    '#description' => t('Configure Drush settings.'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE
  );

  $form['do_drush']['do_drush_enabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable Drush support'),
    '#default_value' => variable_get('do_drush_enabled', 0),
  );

  $form['do_drush']['do_drush_php'] = array(
    '#type' => 'textfield',
    '#title' => t('PHP Path'),
    '#description' => t('The absolute path to PHP.'),
    '#default_value' => variable_get('do_drush_php', 'php'),
    '#required' => FALSE,
  );
  $form['do_drush']['do_drush_drush'] = array(
    '#type' => 'textfield',
    '#title' => t('Drush Path'),
    '#description' => t('The absolute path to <em>drush.php</em>.'),
    '#default_value' => variable_get('do_drush_drush', ''),
    '#required' => FALSE,
  );
  $form['do_drush']['do_drush_root'] = array(
    '#type' => 'textfield',
    '#title' => t('Drupal Path'),
    '#description' => t('The absolute path to the Drupal installation to use Drush on.'),
    '#default_value' => variable_get('do_drush_root', dirname(realpath('index.php'))),
    '#required' => FALSE,
  );

  $form['do_history'] = array(
    '#type' => 'fieldset',
    '#title' => t('History'),
    '#description' => t('Configure settings related to the history of executed commands.'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE
  );

  $form['do_history']['do_history_enable'] = array(
    '#type' => 'checkbox',
    '#name' => 'do_history_enable',
    '#title' => t('Enable history of executed commands'),
    '#default_value' => variable_get('do_history_enable', TRUE)
  );

  $form['do_history']['do_history_limit'] = array(
    '#type' => 'select',
    '#name' => 'do_history_limit',
    '#title' => t('Discard log entries above the following row limit'),
    '#options' => drupal_map_assoc(array(100, 1000, 10000, 100000, 1000000)),
    '#description' => t('The maximum number of rows to keep in the database log. Older entries will be automatically discarded. (Requires a correctly configured <a href="@cron">cron maintenance task</a>.)', array('@cron' => url('admin/reports/status'))),
    '#default_value' => variable_get('do_history_limit', 1000)
  );

  $form['#submit'][] = 'do_form_settings_submit';

  return system_settings_form($form);
}

/**
 * Submit handler for do_form_settings
 * Make sure that the correct character code is saved
 */
function do_form_settings_submit($form, &$form_state) {
  if (!is_numeric($form_state['values']['do_trigger'])) {
    $special_keys = array('esc' => 27, 'tab' => 9, 'space' => 32, 'return' => 13, 'enter' => 13, 'backspace' => 8, 'scrolllock' => 145, 'scroll_lock' => 145, 'scroll' => 145, 'capslock' => 20, 'caps_lock' => 20, 'caps' => 20, 'numlock' => 144, 'num_lock' => 144, 'num' => 144, 'pause' => 19, 'break' => 19, 'insert' => 45, 'home' => 36, 'delete' => 46, 'end' => 35, 'pageup' => 33, 'page_up' => 33, 'pu' => 33,
      'pagedown' => 34, 'page_down' => 34, 'pd' => 34, 'left' => 37, 'up' => 38, 'right' => 39, 'down' => 40, 'f1' => 112, 'f2' => 113, 'f3' => 114, 'f4' => 115, 'f5' => 116, 'f6' => 117, 'f7' => 118, 'f8' => 119, 'f9' => 120, 'f10' => 121, 'f11' => 122, 'f12' => 123);

    if (in_array($form_state['values']['do_trigger'], array_keys($special_keys))) {
      $form_state['values']['do_trigger'] = $special_keys[$form_state['values']['do_trigger']];
    } else {
      $form_state['values']['do_trigger'] = ord($form_state['values']['do_trigger']);
    }
  }
}

/**
 * Alias settings page
 */
function do_form_aliases() {
  $header = array(
    array(
      'data' => t('Name'),
      'field' => 'name',
      'sort' => 'asc'
    ),
    array(
      'data' => t('Command'),
      'field' => 'command'
    ),
    array(
      'data' => t('Parameters'),
      'field' => 'params'
    )
  );

  $resource = pager_query("SELECT name, command, params FROM {do_alias}" . tablesort_sql($header), $limit = 30);
  $data = array();

  while ($row = db_fetch_array($resource)) {
    $data[] = $row;
  }

  if (empty($data)) {
    drupal_set_message(t('There are no aliases defined!'), 'warning');
    return array();
  }

  $table = theme('table', $header, $data);
  $table .= theme('pager');

  $form = array();

  $form['do_clear'] = array(
    '#type' => 'submit',
    '#value' => t('Clear aliases'),
    '#name' => 'clear',
    '#weight' => 0,
    '#prefix' => '<div>',
    '#suffix' => '</div>',
  );

  $form['do_aliases'] = array(
    '#value' => $table,
    '#weight' => 1
  );

  return $form;
}

/**
 * Submit handler for aliases form
 */
function do_form_aliases_submit() {
  db_query("TRUNCATE TABLE {do_alias}");
  drupal_set_message(t('Aliases cleared!'));
}

/**
 * History settings form
 */
function do_form_history() {
  $header = array(
    array(
      'data' => t('Id'),
      'field' => 'id',
      'sort' => 'desc'
    ),
    array(
      'data' => t('Command'),
      'field' => 'command',
    ),
    array(
      'data' => t('Parameters'),
      'field' => 'params'
    ),
    array(
      'data' => t('Date'),
      'field' => 'date'
    ),
    array(
      'data' => t('User'),
      'field' => 'user'
    ),
  );

  $resource = pager_query("SELECT dh.id, dh.command, dh.params, dh.date, dh.user AS uid, u.name AS user FROM {do_history} dh INNER JOIN {users} u ON dh.user = u.uid" . tablesort_sql($header), $limit = 50);
  $data = array();

  while ($row = db_fetch_array($resource)) {
    $data[] = array(
      'id' => $row['id'],
      'command' => $row['command'],
      'params' => $row['params'],
      'date' => date('Y-m-d H:i:s', $row['date']),
      'user' => l($row['user'], 'user/' . $row['uid'])
    );
  }

  if (empty($data)) {
    drupal_set_message(t('There are no items in the history!'), 'warning');
    return array();
  }

  $table = theme('table', $header, $data);
  $table .= theme('pager');

  $form = array();

  $form['do_clear'] = array(
    '#type' => 'submit',
    '#value' => t('Clear history'),
    '#name' => 'clear',
    '#weight' => 0,
    '#prefix' => '<div>',
    '#suffix' => '</div>',
  );

  $form['do_history'] = array(
    '#value' => $table,
    '#weight' => 1
  );

  return $form;
}

/**
 * Submit handler for history form
 */
function do_form_history_submit() {
  db_query("TRUNCATE TABLE {do_history}");
  drupal_set_message(t('History cleared!'));
}