<?php
/**
 * @file Default Do hooks
 */

/**
 * Clear all caches
 * Implementation taken from admin_menu
 */
function do_flush() {
  // Flush all caches; no need to re-implement this.
  module_load_include('inc', 'system', 'system.admin');
  $form = $form_state = array();
  system_clear_cache_submit($form, $form_state);
  return array('ok' => TRUE, 'msg' => t('Caches cleared'));
}

/**
 * Set an alias
 *
 * @param string $alias
 * @param string $op
 * @param string $param1
 * @param string $param2
 * ...
 * @param string $paramN
 */
function do_alias() {
  $args = func_get_args();
  $alias = array_shift($args);
  $op = array_shift($args);

  if (isset($alias) && isset($op)) {
    $ok = do_set_alias($op, $alias, $args);
    do_cache_rebuild();
    $msg = $ok ? t('Alias created successfully!') : t('Error creating alias');
  } else {
    $ok = FALSE;
    $msg = t('Invalid parameters');
  }

  return array('ok' => $ok, 'msg' => $msg);
}

/**
 * Forwards commands to Drush
 *
 * @param string|array $params
 */
function do_drush($params) {
  $drush_enabled = variable_get('do_drush_enabled', 0);

  if (!$drush_enabled) {
    return array('ok' => FALSE, 'msg' => t('Drush support is disabled'));
  }

  $args = implode('/', (array) $params);
  $php = variable_get('do_drush_php', NULL);
  $drush = variable_get('do_drush_drush', NULL);
  $root = variable_get('do_drush_root', NULL);
  global $base_url;

  if (!isset($php, $drush, $root)) {
    return array('ok' => FALSE, 'msg' => t('You must visit admin/settings/do and configure Drush integration.'));
  }

  $output = array();
  exec("$php $drush -r '$root' -l $base_url" . $args, $output);
  $output = implode(' ', $output);
  // @TODO deal with output prompts
  return array('ok' => TRUE, 'msg' => $output);
}

/**
 * Redirect to a destination
 * @see drupal_goto()
 *
 * @param string|array $destination
 */
function do_goto($destination) {
  $path = implode('/', (array) $destination);

  // @TODO check for valid paths?

  // Allow modules to react to the end of the page request before redirecting.
  // We do not want this while running update.php.
  if (!defined('MAINTENANCE_MODE') || MAINTENANCE_MODE != 'update') {
    module_invoke_all('exit', $url);
  }

  // Even though session_write_close() is registered as a shutdown function, we
  // need all session data written to the database before redirecting.
  session_write_close();

  return array('ok' => TRUE, 'redirect' => $path);
}

/**
 * Prints a message
 * The simplest hook in the book
 */
function do_echo() {
  $args = func_get_args();
  $message = implode(' ', $args);
  return array('ok' => TRUE, 'msg' => $message);
}

/**
 * Print help messages
 */
function do_help() {
  // @todo something
}