<?php
/**
 * @file Do Ajax handler
 */

// load hooks
module_load_include('inc', 'do', 'includes/do.hooks');

/**
 * Ajax request handler
 */
function _do_ajax() {
  $response = array('ok' => FALSE);

  // allow force refresh flag
  if (!isset($_GET['refresh']) || $_GET['refresh'] != TRUE) {
    $cache = cache_get('do_aliases');
  }

  // ensure we always have information to work from
  if (!$cache) {
    $cache = do_cache_rebuild();
  }

  // for the sake of convenience
  extract($cache->data);

  // command resolution
  if (isset($_POST['op'])) {
    $args = explode(' ', $_POST['op']);
    $op = array_shift($args);

    // first check user aliases
    while (array_key_exists($op, $aliases['alias'])) {
      $op = $aliases['alias'][$op];
    }

    // user aliases can alias a module alias,
    // so we continue with the resolved alias

    // check default do aliases
    if (array_key_exists($op, $aliases['do'])) {
      $do = $aliases['do'][$op];
    // finally contrib aliases
    } else if (array_key_exists($op, $aliases['contrib'])) {
      // found a module hook, get second argument
      $op2 = array_shift($args);
      if (array_key_exists($op2, $aliases['contrib'][$op])) {
        $do = $aliases['contrib'][$op][$op2];
      } else {
        $pos = array_search($op2, $aliases['contrib'][$op]);
        if ($pos !== FALSE) {
          $do = $aliases['contrib'][$op][$pos];
        }
      }
    }

    // if we still have nothing assume its a path
    if (!isset($do)) {
      $do = 'do_goto';
      // put $op back onto the start of $args
      array_unshift($args, $op);
    }

    // check it exists before calling
    if (function_exists($do)) {
      $hook_return = call_user_func_array($do, $args);
      $response = array_merge($response, (array) $hook_return);

      if (variable_get('do_history_set_enable', TRUE)) {
        do_history_set($do, $args);
      }
    } else {
      $response['msg'] = t('Invalid command: @command', array('@command' => $do));
    }
  }

  if (!isset($response['msg'])) {
    $response['msg'] = ($response['ok'] === FALSE) ? t('WTF?!') : t('Command executed successfully!');
  }

  if (!isset($_GET['debug']) || $_GET['debug'] != TRUE) {
    drupal_json($response);
  } else {
    if (function_exists('krumo')) {
      krumo($response);
    } else {
      var_dump($response);
    }
  }

  die;
}

/**
 * Autocomplete handler
 *
 * @param string $str
 * @TODO improve & finish this
 */
function _do_autocomplete($str) {
  // we now have cached info, probably better than querying alias table directly every time
  $query = "SELECT command FROM do_alias WHERE name LIKE '%%%s%%' ORDER BY name ASC";
  $resource = db_query($query, $op);
  $results = array();

  while ($row = db_fetch_array($result)) {
    $results[] = $row;
  }

  $response = array(
    'ok' => TRUE,
    'data' => $results
  );

  drupal_json($response);
  die;
}

/**
 * Return user-defined aliases
 *
 * @return array
 */
function do_user_alias() {
  // @TODO order by most used (history)?
  $result = array();
  $resource = db_query("SELECT name, command FROM {do_alias} ORDER BY name ASC");
  while ($row = db_fetch_array($resource)) {
    $result[$row['name']] = $row['command'];
  }
  return $result;
}

/**
 * Get command by alias
 *
 * @param string $op
 * @return string|FALSE
 */
function do_get_alias($op) {
  $query = "SELECT command FROM {do_alias} WHERE name = '%s'";
  return db_result(db_query($query, $op));
}

/**
 * Set an alias for a command
 *
 * @param string $op
 * @param string $alias
 * @param array $params
 * @return bool
 */
function do_set_alias($op, $alias, $params = array()) {
  $params = implode(' ', $params);
  $query = "REPLACE INTO {do_alias} SET name = '%s', command = '%s', params = '%s'";
  return db_query($query, $op, $alias, $params) !== false;
}

/**
 * Get events from history
 * @return array
 */
function do_history_get() {
  $query = "SELECT * FROM {do_history} ORDER BY id ASC";
  $resource = db_query($query);
  $result = array();

  while ($row = db_fetch_array($resource)) {
    $result[] = $row;
  }

  return $result;
}

/**
 * Add event to history
 *
 * @param string
 * @param array|string
 * @return bool
 */
function do_history_set($op, $params) {
  global $user;

  if (is_array($params)) {
    $params = implode(' ', $params);
  }

  $query = "INSERT INTO {do_history} (command, params, date, user) VALUES ('%s', '%s', %d, %d)";
  $return = db_query($query, $op, $params, time(), $user->uid);
  return ($return === FALSE) ? FALSE : TRUE;
}

/**
 * Rebuilds Do's cache
 */
function do_cache_rebuild() {
  $hooks = do_invoked_hooks();
  $do = $hooks['do'];
  unset($hooks['do']);

  // add do aliases
  $aliases = array('do' => $do, 'contrib' => array());

  // add get user-defined aliases
  $aliases['alias'] = do_user_alias();

  // add module-defined aliases
  foreach ($hooks as $hook => $functions) {
    $aliases['contrib'][$hook] = $functions;
  }

  // set cache for next requests
  cache_set('do_aliases', array('aliases' => $aliases), 'cache', CACHE_TEMPORARY);

  $return = array('data' => array('aliases' => $aliases));
  return (object) $return;
}

/**
 * Return a list of hooks defined per module
 *
 * @return array
 */
function do_invoked_hooks() {
  $return = array();

  foreach (module_implements('do') as $name) {
    $return[$name] = array();
    $functions = module_invoke($name, 'do');

    foreach ($functions as $k => $v) {
      if (is_numeric($k)) {
        $k = $v;
        $v = "{$name}_{$v}";
      }
      $return[$name][$k] = $v;
    }
  }

  return $return;
}
