/**
 * @file Do core & events
 */

Drupal.Do = Drupal.Do || { overlay: null, timer: null, windowWidth: 0 };

Drupal.Do.open = function() {
  if (!Drupal.Do.overlay) {
    Drupal.Do.init();
  }

  Drupal.Do.overlay.show();
  $('#do_input', Drupal.Do.overlay).focus();
};

Drupal.Do.close = function() {
  $('#do_input', Drupal.Do.overlay).val('');
  Drupal.Do.overlay.hide();
};

Drupal.Do.msg = function(msg, ok) {
  $msg = $('#do_msg', Drupal.Do.overlay);

  if (ok === false) {
    $msg.addClass('error');
  }

  $msg.html(msg).css('width', $msg.width()).slideDown(200, function() {
    setTimeout(function() {
      $msg.slideUp(150, function() {
        $msg.removeClass('error');
      });
    }, 1000);
  });
};

// wrapper to avoid flooding & smooth out stuff
Drupal.Do.autocomplete = function(string) {
  if (Drupal.Do.timer) {
    clearTimeout(Drupal.Do.timer);
  }

  // lookup occurs 200ms after last keystroke
  Drupal.Do.timer = setTimeout(function(){
    Drupal.Do.lookup(string);
  }, 200);
};

Drupal.Do.lookup = function(string) {
//  var serial = 'string=' + string;
//  $.post('/js/do/autocomplete', serial, function(response) {
//    console.log(response);
//    // @todo handle response
//  }, 'json');
}

Drupal.Do.loader = function(show) {
  if (show) {
    $('#do_input', Drupal.Do.overlay).addClass('loading');
  } else {
    $('#do_input', Drupal.Do.overlay).removeClass('loading');
  }
}

Drupal.Do.execute = function() {
  var $input = $('#do_input', Drupal.Do.overlay),
      val = $input.val(),
      msg;

  if (val !== '') {
    Drupal.Do.loader(true);
    $.post('/js/do/ajax', 'op=' + val, function(response) {
      if (response.redirect) {
        if (response.redirect === true) {
          location.reload(true);
        } else {
          window.location = Drupal.settings.basePath + response.redirect;
        }
      }

      Drupal.Do.loader(false);

      if (response.ok === true) {
        $input.val('');
        msg = response.msg || Drupal.t('Command executed successfully');
      } else {
        msg = response.msg || Drupal.t('Doh!');
      }

      Drupal.Do.msg(msg, response.ok);
      return false;
    }, 'json');
  }
};

Drupal.Do.init = function(context) {
  Drupal.Do.overlay = $('<div id="do_overlay" />');
  var $modal = $('<div id="do_modal" />'),
      $status = $('<div id="do_status"><div id="do_msg" /></div>'),
      $input = $('<input type="text" id="do_input" />');

  $input.bind('keyup', function(e) {
    var code = e.keyCode ? e.keyCode : e.which;
    if (code === 13) {
      Drupal.Do.execute();
    }else if (code === 27) {
      Drupal.Do.close();
    } else {
      Drupal.Do.autocomplete($input.val());
    }
  });

  $modal.append($status, $input);
  Drupal.Do.overlay.append($modal).click(function() {
    $input.focus();
  });
  Drupal.Do.windowWidth = $(window).width();
  $('body', context).append(Drupal.Do.overlay);
};

/*
 * Do Events
 */

Drupal.behaviors.do_trigger = function(context) {
  // bind keyboard shortcut
  $(document, context).bind('keyup', function(e) {
    var code = e.keyCode ? e.keyCode : e.which;
    if (code == Drupal.settings.Do.trigger) {
      var key = Drupal.settings.Do.modifier + 'Key';
      // nifty trick to avoid a bunch of if statements - this will probably break in some browsers
      if (e[key] === true) {
        Drupal.Do.open();
        e.cancelBubble = true; // for IE
        e.returnValue = false;
        if (e.stopPropagation) {
          e.stopPropagation();
          e.preventDefault();
        }
        return false;
      }
    }
  });
};